@@ -101,8 +101,11 @@ extern SEXP panyR(SEXP na, SEXP args);
 extern SEXP pcountR(SEXP x, SEXP args);
 extern SEXP pcountNAR(SEXP args);
 extern SEXP pfirstR(SEXP last, SEXP args);
+extern SEXP fpminR(SEXP na, SEXP args);
+extern SEXP fpmaxR(SEXP na, SEXP args);
 extern SEXP pmeanR(SEXP na, SEXP args);
 extern SEXP pprodR(SEXP na, SEXP args);
+extern SEXP prangeR(SEXP na, SEXP args);
 extern SEXP psumR(SEXP na, SEXP args);
 extern SEXP setlevelsR(SEXP x, SEXP old_lvl, SEXP new_lvl, SEXP skip_absent);
 extern SEXP subSetColDataFrame(SEXP df, SEXP str);
